
class LunaDatasetNotSupportedException(Exception):

    def __init__(self, message):
        self.message = message

class LunaInvalidConfigException(Exception):

    missing_property_list = []

    def __init__(self, message, missing_property_list=[]):
        self.message = message
        self.missing_property_list = missing_property_list


from azureml.core import Workspace, Run, Experiment
from azureml.core.compute import ComputeTarget, AmlCompute
from azureml.core.environment import Environment
from azureml.core.model import Model, InferenceConfig
from azureml.core.runconfig import RunConfiguration
from azureml.core.webservice import AksWebservice, AciWebservice, Webservice
from azureml.pipeline.core.graph import PipelineParameter
from azureml.pipeline.steps import PythonScriptStep
from azureml.pipeline.core import Pipeline
from azureml.core.webservice import Webservice
from azureml.core.webservice.aci import AciWebservice
from azureml.core.webservice.aks import AksWebservice

import yaml
import io
import argparse
import json
import os
import mlflow
import time

class ProjectUtils(object):

    luna_config={}
    run_mode='azureml'

    def __init__(self, luna_config_file='luna_config.yml', run_mode='default'):
        with open(luna_config_file) as file:
            self.luna_config = yaml.full_load(file)
        
        if run_mode == 'default':
            run_mode = self.luna_config['run_mode']

    def IfRunInAzureML(self):
        try:
            run = Run.get_context(allow_offline=False)
        except:
            return False
        
        return run is not None

    def RegisterModel(self, model_path, description, args):

        if self.IfRunInAzureML():
            experiment = Run.get_context(allow_offline=False).experiment
            ws = experiment.workspace

            Model.register(model_path = model_path,
                        model_name = args.operationId,
                        description = description,
                        workspace = ws,
                        tags={'userId': args.userId, 
                            'productName': args.productName, 
                            'deploymentName': args.deploymentName, 
                            'apiVersion':args.apiVersion,
                            'subscriptionId':args.subscriptionId})
        else:
            print('run on MLflow')
            mlFlowRun = mlflow.active_run()
            print(mlFlowRun)
            if mlFlowRun:
                mlflow.log_artifacts(model_path, model_path)
                model_uri = "runs:/{run_id}/{artifact_path}".format(run_id=mlFlowRun.info.run_id, artifact_path=model_path)
                print(model_uri)
                result = mlflow.register_model(
                    model_uri,
                    args.operationId
                )
                print(result)

    
    def DownloadModel(self, model_id, model_path):
        if self.IfRunInAzureML():
            experiment = Run.get_context(allow_offline=False).experiment
            ws = experiment.workspace
            model = Model(ws, model_id)
            model.download(target_dir = model_path, exist_ok=True)

    def DownloadOutputFromPredecessorOperation(self, predecessor_op_id, folder_path):
        return

    def GetModelPath(self):
        # get model path if the model is deployed using AML
        if os.getenv('AZUREML_MODEL_DIR'):
            return os.getenv('AZUREML_MODEL_DIR')

    def ParseArguments(self, run_type):
        parser = argparse.ArgumentParser(run_type)

        parser.add_argument("--runMode", type=str, help="run mode, support azureML or MLflow")
        parser.add_argument("--userInput", type=str, help="input data")
        parser.add_argument("--modelId", type=str, help="model key")

        if run_type == 'inference':
            parser.add_argument("--operationId", type=str, help="run id")
        else:
            parser.add_argument("--userId", type=str, help="user id")
            parser.add_argument("--productName", type=str, help="product name")
            parser.add_argument("--deploymentName", type=str, help="deployment name")
            parser.add_argument("--apiVersion", type=str, help="api version")
            parser.add_argument("--subscriptionId", type=str, help="subscription id")

            if run_type == 'deployment':
                parser.add_argument("--endpointId", type=str, help="endpoint id")

        args = parser.parse_args()
        userInput = json.loads(args.userInput)

        return args, userInput

    def GetDeploymentConfig(self, dns_name_label,tags):
        workspace_full_path = os.path.join(self.luna_config['azureml']['workspace_config_path'], self.luna_config['azureml']['workspace_config_file_name'])
        with open(workspace_full_path) as file:
            documents = json.load(file)
            deployment_target = documents['DeploymentTarget']
            aks_cluster = documents['AksCluster']


        with open(self.luna_config['deploy_config']) as file:
            documents = yaml.full_load(file)

            if deployment_target == 'aci':
                deployment_config = AciWebservice.deploy_configuration()
                deployment_config.__dict__.update(documents['azureContainerInstance'])
                deployment_config.dns_name_label = dns_name_label
            elif deployment_target == 'aks':
                deployment_config = AksWebservice.deploy_configuration()
                deployment_config.__dict__.update(documents['kubernetes'])
                deployment_config.compute_target_name = aks_cluster
                deployment_config.namespace = dns_name_label

            deployment_config.tags = tags
        return deployment_config

    def DeployModel(self):

        args, userInput = self.ParseArguments("deployment")

        dns_name_label = userInput["dns_name_label"]
        model_id = args.predecessorOperationId
        endpoint_id = args.operationId

        print(dns_name_label)

        ## If it is running in AML context
        if self.IfRunInAzureML():
            experiment = Run.get_context(allow_offline=False).experiment
            ws = experiment.workspace
            model = Model(ws, model_id)

            myenv = Environment.from_conda_specification('scoring', self.luna_config['conda_env'])

            inference_config = InferenceConfig(entry_script=self.luna_config['code']['inference_entry_script'], source_directory = os.getcwd(), environment=myenv)

            deployment_config = self.GetDeploymentConfig(
                dns_name_label,
                tags={'userId': args.userId, 
                    'productName': args.productName, 
                    'deploymentName': args.deploymentName, 
                    'apiVersion':args.apiVersion,
                    'subscriptionId':args.subscriptionId})
            
            service = Model.deploy(ws, endpoint_id, [model], inference_config, deployment_config)
            service.wait_for_deployment(show_output = True)

    def WaitForRunCompletionByTags(self, experiment_name, tags, show_output = True):
        
        time.sleep(5)
        ws = Workspace.from_config(path=self.luna_config['azureml']['workspace_config_path'], _file_name=self.luna_config['azureml']['workspace_config_file_name'])
        exp = Experiment(ws, experiment_name)
        runs = exp.get_runs(type='azureml.PipelineRun', tags=tags)
        run = next(runs)
        run.wait_for_completion(show_output = show_output)

    def WaitForRunCompletion(self, run_id, experiment_name):
        
        time.sleep(5)
        ws = Workspace.from_config(path=self.luna_config['azureml']['workspace_config_path'], _file_name=self.luna_config['azureml']['workspace_config_file_name'])
        
        exp = Experiment(ws, experiment_name)

        run = Run(exp, run_id)
        run.wait_for_completion(show_output = False)

    def GetServiceEndpoint(self, endpoint_id):
        ws = Workspace.from_config(path=self.luna_config['azureml']['workspace_config_path'], _file_name=self.luna_config['azureml']['workspace_config_file_name'])
        return Webservice(ws, endpoint_id)
    
    def GetAciServiceEndpoint(self, endpoint_id):
        ws = Workspace.from_config(path=self.luna_config['azureml']['workspace_config_path'], _file_name=self.luna_config['azureml']['workspace_config_file_name'])
        return AciWebservice(ws, endpoint_id)

    def GetAksServiceEndpoint(self, endpoint_id):
        ws = Workspace.from_config(path=self.luna_config['azureml']['workspace_config_path'], _file_name=self.luna_config['azureml']['workspace_config_file_name'])
        return AksWebservice(ws, endpoint_id)

    def RunProject(self, entry_point, experiment_name, parameters, tags, azureml_workspace=None, compute_cluster="default"):

        if not azureml_workspace:
            azureml_workspace = Workspace.from_config(path=self.luna_config['azureml']['workspace_config_path'], _file_name=self.luna_config['azureml']['workspace_config_file_name'])

        if azureml_workspace:
            run_config = RunConfiguration.load(self.luna_config['azureml']['run_config'])

            if compute_cluster == "default":
                workspace_full_path = os.path.join(self.luna_config['azureml']['workspace_config_path'], self.luna_config['azureml']['workspace_config_file_name'])
                with open(workspace_full_path) as file:
                    documents = json.load(file)
                    aml_compute = documents['AmlCompute']
                    run_config.target = aml_compute
            else:
                run_config.target = compute_cluster

            arguments = self.GetPipelineArguments(self.luna_config['MLproject'], entry_point, parameters)

            entry_point_script = self.GetEntryPointScriptPath(self.luna_config['MLproject'], entry_point)

            trainStep = PythonScriptStep(
                script_name=entry_point_script,
                arguments=arguments,
                inputs=[],
                outputs=[],
                source_directory=os.getcwd(),
                runconfig=run_config,
                allow_reuse=False
            )

            pipeline = Pipeline(workspace=azureml_workspace, steps=[trainStep])

            experiment = Experiment(azureml_workspace, experiment_name)
            pipeline_run = experiment.submit(pipeline, tags=tags)
            return pipeline_run.id
        
        else:
            return '00000000-0000-0000-0000-000000000000'

    def PublishAMLPipeline(self, entry_point, name, description, azureml_workspace=None, parameters={}, compute_cluster="default"):

        if not azureml_workspace:
            azureml_workspace = Workspace.from_config(path=self.luna_config['azureml']['workspace_config_path'], _file_name=self.luna_config['azureml']['workspace_config_file_name'])

        if azureml_workspace:
            run_config = RunConfiguration.load(self.luna_config['azureml']['run_config'])
            
            if compute_cluster == "default":
                workspace_full_path = os.path.join(self.luna_config['azureml']['workspace_config_path'], self.luna_config['azureml']['workspace_config_file_name'])
                with open(workspace_full_path) as file:
                    documents = json.load(file)
                    aml_compute = documents['AmlCompute']
                    run_config.target = aml_compute
            else:
                run_config.target = compute_cluster

            arguments = self.GetPipelineArguments(self.luna_config['MLproject'], entry_point, parameters)
            entry_point_script = self.GetEntryPointScriptPath(self.luna_config['MLproject'], entry_point)
            trainStep = PythonScriptStep(
                script_name=entry_point_script,
                arguments=arguments,
                inputs=[],
                outputs=[],
                source_directory=os.getcwd(),
                runconfig=run_config
            )

            pipeline = Pipeline(workspace=azureml_workspace, steps=[trainStep])
            published_pipeline = pipeline.publish(name=name, description=description)
            return published_pipeline.endpoint

    def GetPipelineArguments(self, mlproject_file_path, run_type, parameters):
        with open(mlproject_file_path) as file:
            documents = yaml.full_load(file)
            arguments = []
            for param in documents['entry_points'][run_type]['parameters']:
                argumentName = '--' + param
                arguments.append(argumentName)
                # get input parameter values, set default value if not provided
                if param in parameters:
                    pipelineParam = PipelineParameter(name=param, default_value=parameters[param])
                else:
                    pipelineParam = PipelineParameter(name=param, default_value=documents['entry_points'][run_type]['parameters'][param]['default'])
                arguments.append(pipelineParam)
            return arguments

    def GetEntryPointScriptPath(self, mlproject_file_path, entry_point):
        with open(mlproject_file_path) as file:
            documents = yaml.full_load(file)
            command = documents['entry_points'][entry_point]['command']
            script_path = command.split(" ")[1]
            return script_path

    def GetOperationNameByVerb(self, operationVerb):
        
        with open(self.luna_config['MLproject']) as file:
            documents = yaml.full_load(file)
            for operation in documents['entry_points']:
                if documents['entry_points'][operation]['verb'] == operationVerb:
                    return operation

        return None


    def GetOperationNameByNoun(self, operationNoun):
        
        with open(self.luna_config['MLproject']) as file:
            documents = yaml.full_load(file)
            for operation in documents['entry_points']:
                if documents['entry_points'][operation]['noun'] == operationNoun:
                    return operation

        return None


    def GetOutputType(self, operationName):
        
        with open(self.luna_config['MLproject']) as file:
            documents = yaml.full_load(file)
            return documents['entry_points'][operationName]['output']


from abc import ABCMeta, abstractmethod

class LunaDataset(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def to_pandas_dataframe(self):
        """
        Read tabular data from the dataset and return a pandas dataframe
        """

    @abstractmethod
    def write(self, dataframe):
        """
        Write pandas dataframe to the dataset as tabular data
        """

    @abstractmethod
    def download(self, path):
        """
        Download the file(s) from the dataset
        """

    @abstractmethod
    def upload(self, path):
        """
        Upload file(s) to the dataset
        """



from luna.dataset.luna_dataset import LunaDataset
from luna.dataset.exceptions import LunaDatasetNotSupportedException, LunaInvalidConfigException
import pandas as pd 
import requests
import tempfile
import os

class LunaTabularDataset(LunaDataset):

    def download(self):
        """
        Download function is not supported in LunaTabularDataset. Use LunaFileDataset instead.
        """
        raise LunaDatasetNotSupportedException("Download function is not supported in LunaTabularDataset. Use LunaFileDataset instead.")

    def upload(self, path):
        """
        Upload function is not supported in LunaTabularDataset. Use LunaFileDataset instead.
        """
        raise LunaDatasetNotSupportedException("Upload function is not supported in LunaTabularDataset. Use LunaFileDataset instead.")

    @staticmethod
    def create(config):
        """
        Create a LunaTabularDataset from config dict
        """
        if "url" in config:
            return LunaUrlTabularDataset(config["url"])
        elif "azureml_dataset_name" in config:
            return LunaAzureMLTabularDataset(config["azureml_dataset_name"])
        elif "sql_connection_string" in config:
            table_name = ""
            query = ""
            if "table_name" in config:
                table_name = config["table_name"]
            if "query" in config:
                query = config["query"]
            # Need to specify either table name or query
            if table_name == "" and query == "":
                raise LunaInvalidConfigException("Either table_name or query property needs to be provided for LunaSQLTabularDataset.", ["table_name", "query"])
            return LunaSQLTabularDataset(config["sql_connection_string"], table_name = table_name, query = query)
        else:
            raise LunaDatasetNotSupportedException("The config does not match any tabular dataset.")

class LunaAzureMLTabularDataset(LunaTabularDataset):
    """
    Read tabular data from Azure ML tabular dataset.
    Only supported when running in an Azure ML workspace.
    """

    _azureml_dataset_name = ""

    def to_pandas_dataframe(self):
        """
        Read tabular data from Azure ML tabular dataset and return as a Pandas dataframe
        """
        pass
    
    def write(self, df):
        """
        Write function is not supported in LunaAzureMLTabularDataset.
        """
        raise LunaDatasetNotSupportedException("Write function is not supported in LunaAzureMLTabularDataset.")
    
    def __init__(self, azureml_dataset_name):
        self._azureml_dataset_name = azureml_dataset_name

class LunaSQLTabularDataset(LunaTabularDataset):

    _connection_string = ""
    _table_name = ""
    _query = ""

    def to_pandas_dataframe(self):
        """
        Read tabular data from SQL Server and return as a Pandas dataframe
        """
        pass
    
    def write(self, df):
        """
        Write tabular data to SQL Server
        """
        pass

    def __init__(self, connection_string, table_name='', query=''):
        self._connection_string = connection_string
        self._table_name = table_name
        self._query = query


class LunaUrlTabularDataset(LunaTabularDataset):

    _url = ""

    def to_pandas_dataframe(self, delimiter=",", header="infer", names=None, skiprows=None, nrows=None, index_col=None, low_memory=True):
        """
        Read tabular data from an csv file url and return as a Pandas dataframe
        """

        return pd.read_csv(self._url, delimiter=delimiter, header=header, names=names, skiprows=skiprows, nrows=nrows, index_col=index_col, low_memory=low_memory)
    
    def write(self, df, header=False):
        """
        Write tabular data to an csv file url
        Right now it only works for a public Azure block blob url or private blob url with sas key
        """
        with tempfile.TemporaryDirectory() as tmp:
            temp_filename = os.path.join(tmp, "tmp_result.csv")
            with open(temp_filename, "wt") as temp_file:
                df.to_csv(temp_file, header=header)

            with open(temp_filename , 'rb') as fh:
                requests.put(self._url,
                                data=fh,
                                headers={
                                            'content-type': 'text/csv',
                                            'x-ms-blob-type': 'BlockBlob'
                                        }
                                )

        return

    def __init__(self, url):
        self._url = url



from luna.dataset.luna_dataset import LunaDataset
from luna.dataset.exceptions import LunaDatasetNotSupportedException, LunaInvalidConfigException

class LunaFileDataset(LunaDataset):

    def to_pandas_dataframe(self):
        """
        to_pandas_dataframe function is not supported in LunaFileDataset. Use LunaTabularDataset instead.
        """
        raise LunaDatasetNotSupportedException("to_pandas_dataframe function is not supported in LunaTabularDataset. Use LunaFileDataset instead.")

    def write(self, df):
        """
        Write function is not supported in LunaFileDataset. Use LunaTabularDataset instead.
        """
        raise LunaDatasetNotSupportedException("Write function is not supported in LunaTabularDataset. Use LunaFileDataset instead.")

    @staticmethod
    def create(config):
        """
        Create a LunaFileDataset from config dict
        """
        if "url" in config:
            return LunaUrlFileDataset(config["url"])
        elif "azureml_dataset_name" in config:
            return LunaAzureMLFileDataset(config["azureml_dataset_name"])
        elif "storage_account_name" in config and "path" in config and "storage_container_name" in config:
            sas_key = ""
            storage_account_key = ""
            if "storage_account_key" in config:
                storage_account_key = config["storage_account_key"]
            if "sas_key" in config:
                sas_key = config["sas_key"]
            # Need to specify either table name or query
            if sas_key == "" and storage_account_key == "":
                raise LunaInvalidConfigException("Either storage_account_key or sas_key property needs to be provided for LunaAzureStorageFileDataset.", ["storage_account_key", "sas_key"])
            return LunaAzureStorageFileDataset(config["storage_account_name"], config["storage_container_name"], config["path"], storage_account_key = storage_account_key, sas_key = sas_key)
        else:
            raise LunaDatasetNotSupportedException("The config does not match any tabular dataset.")

class LunaAzureMLFileDataset(LunaFileDataset):
    """
    Download file(s) from Azure ML file dataset.
    Only supported when running in an Azure ML workspace.
    """

    _azureml_dataset_name = ""

    def download(self, path):
        """
        Download file(s) from Azure ML file dataset
        """
        pass

    def upload(self, path):
        """
        Upload function is not supported in LunaAzureMLFileDataset 
        """
        raise LunaDatasetNotSupportedException("Upload function is not supported in LunaAzureMLFileDataset")

    def __init__(self, azureml_dataset_name):
        self._azureml_dataset_name = azureml_dataset_name

class LunaUrlFileDataset(LunaFileDataset):

    _url = ""
    def download(self, path):
        """
        Download a file from specifed url
        """
        pass

    def upload(self, path):
        """
        Upload a file to the specified url
        """
        pass

    def __init__(self, url):
        self._url = url

class LunaAzureStorageFileDataset(LunaFileDataset):

    _storage_account_name = ""
    _storage_account_key = ""
    _storage_container_name = ""
    _sas_key = ""
    _path = ""

    def download(self, path):
        """
        Download file(s) from an Azure storage account
        """
        pass

    def upload(self, path):
        """
        Upload file(s) to Azure storage account
        """
        pass

    def __init__(self, storage_account_name, storage_container_name, path, storage_account_key, sas_key):
        self._storage_account_name = storage_account_name
        self._storage_container_name = storage_container_name
        self._storage_account_key = storage_account_key
        self._sas_key = sas_key
        self._path = path



from azureml.core import Workspace, Run, Experiment
from azureml.core.compute import ComputeTarget, AmlCompute
from azureml.core.environment import Environment
from azureml.core.model import Model, InferenceConfig
from azureml.core.runconfig import RunConfiguration
from azureml.core.webservice import AksWebservice, AciWebservice, Webservice
from azureml.pipeline.core.graph import PipelineParameter
from azureml.pipeline.steps import PythonScriptStep
from azureml.pipeline.core import Pipeline

from luna.baseLunaUtils import BaseLunaUtils
from luna.logging.azuremlLunaLogger import AzureMLLunaLogger

import os
import yaml
import tempfile
import json

class AzureMLLunaUtils(BaseLunaUtils):

    def Init(self, luna_config, run_mode, args, userInput):
        super().Init(luna_config, run_mode, args, userInput)
        self._logger = AzureMLLunaLogger()

    def GetAMLWorkspace(self):
        try:
            run = Run.get_context(allow_offline=False)
            return run.experiment.workspace
        except:
            return Workspace.from_config(path=self._luna_config["azureml"]["workspace_config_path"], 
                _file_name=self._luna_config["azureml"]["workspace_config_file_name"])
    
    def RegisterModel(self, model_path, description, luna_python_model=None):
        ws = self.GetAMLWorkspace()

        Model.register(model_path = model_path,
                       model_name = self._args.operationId,
                       description = description,
                       workspace = ws,
                       tags={'userId': self._args.userId, 
                        'productName': self._args.productName, 
                        'deploymentName': self._args.deploymentName, 
                        'apiVersion':self._args.apiVersion,
                        'subscriptionId':self._args.subscriptionId,
                        'modelId': self._args.operationId})

    def GetDeploymentConfig(self, tags, deployment_target='default', aks_cluster='default'):

        # Read default deployment target and aks cluster info from the config files
        
        workspace_full_path = os.path.join(self._luna_config['azureml']['workspace_config_path'], self._luna_config['azureml']['workspace_config_file_name'])
        with open(workspace_full_path) as file:
            documents = json.load(file)
            if deployment_target == 'default':
                deployment_target = documents['DeploymentTarget']
            if aks_cluster == 'default' and deployment_target == 'aks':
                aks_cluster = documents['AksCluster']

        with open(self._luna_config['deploy_config']) as file:
            documents = yaml.full_load(file)

            if deployment_target == 'aci':
                deployment_config = AciWebservice.deploy_configuration()
                deployment_config.__dict__.update(documents['azureContainerInstance'])
                deployment_config.dns_name_label = self._userInput["dns_name_label"]
            elif deployment_target == 'aks':
                deployment_config = AksWebservice.deploy_configuration()
                deployment_config.__dict__.update(documents['kubernetes'])
                deployment_config.compute_target_name = aks_cluster
                deployment_config.namespace = self._userInput["dns_name_label"]

            deployment_config.tags = tags
        return deployment_config

    def DeployModel(self):
        
        ws = self.GetAMLWorkspace()
        model = Model(ws, self._args.predecessorOperationId)
        myenv = Environment.from_conda_specification('scoring', self.luna_config['conda_env'])

        inference_config = InferenceConfig(entry_script=self.luna_config['code']['inference_entry_script'], source_directory = os.getcwd(), environment=myenv)

        deployment_config = self.GetDeploymentConfig(
            tags={'userId': self._args.userId, 
                'productName': self._args.productName, 
                'deploymentName': self._args.deploymentName, 
                'apiVersion':self._args.apiVersion,
                'subscriptionId':self._args.subscriptionId,
                'modelId': self._args.predecessorOperationId,
                'endpointId': self._args.operationId},
                deployment_target=self._args.deploymentTarget,
                aks_cluster=self._args.aksCluster)
        
        service = Model.deploy(ws, self._args.operationId, [model], inference_config, deployment_config)
        service.wait_for_deployment(show_output = True)

    def DownloadModel(self, model_path=""):
        ws = self.GetAMLWorkspace()
        model = Model(ws, self._args.predecessorOperationId)
        full_model_path = os.path.join(os.getcwd(), model_path, "models/artifacts")
        
        os.makedirs(full_model_path, exist_ok=True)
        model.download(target_dir = full_model_path, exist_ok=True)
        return full_model_path

    def FindPredecessorRun(self):
        run = Run.get_context(allow_offline=False)
        experiment = run.experiment
        
        tags={'userId': self._args.userId,
              'subscriptionId':self._args.subscriptionId,
              'operationId': self._args.predecessorOperationId}

        runs = experiment.get_runs(type='azureml.PipelineRun', tags=tags)
        try:
            return next(runs)
        except StopIteration:
            return None

    def GetJsonOutputFromPredecessorRun(self):
        """
        Get JSON output from predecessor run
        Return None if the run is not found
        """
        predecessorRun = self.FindPredecessorRun()
        if predecessorRun:
            with tempfile.TemporaryDirectory() as tmp:
                path = os.path.join(tmp, self._args.predecessorOperationId, 'output.json')
                predecessorRun.download_file('/outputs/output.json', path)
                with open(path) as file:
                    return json.load(file)

        return None

    def DownloadOutputFilesFromPredecessorRun(self, targetFolder):
        """
        Download output files from predecessor run
        Return all file names if the run is found. Otherwise, return None
        """
        predecessorRun = self.FindPredecessorRun()
        if predecessorRun:
            childRuns = predecessorRun.get_children()
            try:
                childRun = next(childRuns)
            except StopIteration:
                return None

            if not os.path.exists(targetFolder):
                os.makedirs(targetFolder)
            childRun.download_files(prefix = 'outputs', output_directory=targetFolder, append_prefix=False)
            files = childRun.get_file_names()
            return [file for file in files if file.startswith("outputs/")]
        
        return None
        

    def WriteJsonOutput(self, content):
        """
        Write json output to current run
        """
        with tempfile.TemporaryDirectory() as tmp:
            path = os.path.join(tmp, self._args.predecessorOperationId, 'output.json')
            with open(path, 'w') as outfile:
                json.dump(content, outfile)

        self._logger.upload_artifacts(path, 'outputs/output.json')
    
    def UploadOutputFiles(self, sourceFolder):
        """
        Upload files to output of current run
        """
        self._logger.upload_artifacts(sourceFolder, 'outputs')


import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="luna-publish-utils",
    version="0.7.3",
    description="Publishing utilities for project Luna",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/allenwux/ml/src/luna/utils",
    author="Xiaochen Wu",
    author_email="allenwu.x@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["luna", "luna.logging", "luna.dataset"],
    include_package_data=True,
    install_requires=["azureml-sdk", "mlflow", "pandas"],
    entry_points={},
)